from mcp_software_ccopyright.yu.processors import LineFilterBase


class CommentBlockFilter(LineFilterBase):
    """ Filters out everything - when the file is currently in a comment block.
    """

    def filter(self, file, line):
        if file.is_in_comment_block:
            file.found_comment_line()
            return line, True
        return line, False



from mcp.server.fastmcp import FastMCP
import shutil
import os
from mcp_software_ccopyright.yu.project import Project
from mcp_software_ccopyright.yu.processors import FileProcessor
import os
from string import Template

from .crew import report


mcp = FastMCP("mcp-software-ccopyright")

@mcp.tool(description="开始软著相关要求和材料准备， 入参： type: 单人/多人/企业/多个企业/院校/院校合作 ，project_name: 项目名称， code_path: 代码路径，output_dir: 输出目录 ")  
def kickoff(type:str, project_name: str, code_path: str, output_dir: str) -> dict:
    """
    软著相关要求和材料准备
    
    Args:
        type: 单人/多人/企业/多个企业/院校/院校合作
        project_name: 项目名称
        code_path: 代码路径
        output_dir: 输出目录
        
    Returns:
        操作结果消息
    """
    description = """
    """
    
    return report(type, project_name, code_path, output_dir)

        



# def main():
#     """主函数，运行MCP服务器"""
#     try:
#         print("启动 MCP 服务器...")
#         # 运行服务器
#         mcp.run(transport='stdio')
#     except Exception as e:
#         print(f"Error: {e}")

# if __name__ == "__main__":
#     main() 


""" Yu project
"""
from mcp_software_ccopyright.yu.project import Project
from mcp_software_ccopyright.yu.project_info import ProjectInfo
from mcp_software_ccopyright.yu.file_info import FileInfo

__all__ = ['Project', 'ProjectInfo', 'FileInfo']



from mcp_software_ccopyright.yu.processors import LineFilterBase


class HtmlCommentBlockFilter(LineFilterBase):
    """ Filters out html comment blocks with '<!--' and '-->'
    """

    def filter(self, file, line):
        if file.is_in_comment_block:
            file.found_comment_line()
            if line.endswith('-->'):
                file.mark_comment_block_end()
            return line, True
        else:
            if line.startswith('<!--'):
                file.found_comment_line()
                file.mark_comment_block_start()
                if line.endswith('-->'):
                    file.mark_comment_block_end()
                return line, True
        return line, False



class FileInfo(object):
    file_name = ''
    file_extension = ''
    file_path = ''
    is_in_comment_block = False
    blank_lines = 0
    comment_lines = 0
    extracted_lines = 0

    def __init__(self, file_path, file_name, file_extension):
        self.file_path = file_path
        self.file_extension = file_extension
        self.file_name = file_name
        return

    def has_extracted_lines(self):
        return self.extracted_lines > 0

    def extracted_line(self):
        self.extracted_lines += 1
        return

    def found_blank_line(self):
        self.blank_lines += 1
        return

    def found_comment_line(self):
        self.comment_lines += 1
        return

    def mark_comment_block_start(self):
        self.is_in_comment_block = True
        return

    def mark_comment_block_end(self):
        self.is_in_comment_block = False
        return



from mcp_software_ccopyright.yu.processors import LineFilterBase


class BlankLineFilter(LineFilterBase):
    """ Filters out blank lines - lines with only spaces, tabs and line ends. It also strips the line for later processing.
    """

    def filter(self, file, line):
        stripped_line = line.strip(' \t\n')
        if stripped_line == '':
            file.found_blank_line()
            return stripped_line, True
        return stripped_line, False



"""
命令行入口点，用于运行 MCP 服务器
"""
import sys
import logging
from mcp.server.fastmcp import FastMCP
from mcp_software_ccopyright.server import mcp

# 设置日志格式
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)

def main():
    """主函数，运行MCP服务器"""
    try:
        print("启动 MCP ...", file=sys.stderr)
        # 运行服务器
        mcp.run(transport='stdio')
    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)

if __name__ == "__main__":
    main() 


from mcp_software_ccopyright.yu.processors import LineFilterBase


class DoubleSlashCommentFilter(LineFilterBase):
    """ Filters out single line comments which start with '//'
    """

    def filter(self, file, line):
        if line.startswith('//'):
            file.found_comment_line()
            return line, True
        return line, False



import io


class FileProcessorBase(object):
    """ Base class for file processors. The processor for each lanuage should inherit from this class.
    """
    expected_extensions = []
    filters = []

    def __init__(self):
        # by default, processors of all languages will always starts with a blank line filter
        self.filters.append(BlankLineFilter())
        return

    def is_interested_in(self, extension):
        """ Checks whether the filter is interested in the provided file extension.
        """
        for expected in self.expected_extensions:
            if expected == extension:
                return True
        return False

    def process(self, file):
        """ Processes a file and extracts lines out of it.
        """
        with io.open(file.file_path, 'r', encoding='utf-8') as open_file:
            last_line = None
            for original_line in open_file:
                if self.process_line(file, original_line):
                    yield original_line
                    last_line = original_line
        return

    def process_line(self, file, line):
        """ Processes a line, returns true if the line should be extracted.
        """
        for filter in self.filters:
            line, dropped = filter.filter(file, line)
            if dropped:
                return False
        return True

    def add_filter(self, filter):
        self.filters.append(filter)
        return


class LineFilterBase(object):
    """ A filter will process each line, and determine whether each line should be dropped. A filter can also perform any neccessary process on the line and replace the original line.
    """

    def filter(self, line):
        """ Filters a line of code, outputs the filtered content, and a flag whether the line should be dropped.
        """
        return line, False


class FileProcessor(object):

    def __init__(self):
        self.processors = {}
        self.__build_processors()
        return

    def print_processors(self):
        return list(self.processors.keys())

    def has_interest(self, extension):
        return extension in self.processors

    def process(self, file):
        processor = self.__get_cached_processor(file.file_extension)
        for output in processor.process(file):
            yield output

    def __build_processors(self):
        """ Register and cache all supported file processors.
        """
        self.__cache_processor(JsProcessor())
        self.__cache_processor(JavaProcessor())
        self.__cache_processor(PhpProcessor())
        self.__cache_processor(HtmlProcessor())
        self.__cache_processor(CssProcessor())
        self.__cache_processor(SwiftProcessor())
        self.__cache_processor(OCProcessor())
        self.__cache_processor(GoProcessor())
        return

    def __get_cached_processor(self, extension):
        return self.processors[extension]

    def __cache_processor(self, processor):
        if len(processor.expected_extensions) > 0:
            for extension in processor.expected_extensions:
                self.processors[extension] = processor
        return


from mcp_software_ccopyright.yu.processors.blank_line_filter import BlankLineFilter
from mcp_software_ccopyright.yu.processors.comment_block_filter import CommentBlockFilter
from mcp_software_ccopyright.yu.processors.js_processor import JsProcessor
from mcp_software_ccopyright.yu.processors.java_processor import JavaProcessor
from mcp_software_ccopyright.yu.processors.php_processor import PhpProcessor
from mcp_software_ccopyright.yu.processors.html_processor import HtmlProcessor
from mcp_software_ccopyright.yu.processors.css_processor import CssProcessor
from mcp_software_ccopyright.yu.processors.swift_processor import SwiftProcessor
from mcp_software_ccopyright.yu.processors.oc_processor import OCProcessor
from mcp_software_ccopyright.yu.processors.go_processor import GoProcessor



from mcp_software_ccopyright.yu.processors import FileProcessorBase
from mcp_software_ccopyright.yu.processors import BlankLineFilter
from mcp_software_ccopyright.yu.processors.c_style_comment_block_filter import CStyleCommentBlockFilter
from mcp_software_ccopyright.yu.processors.double_slash_comment_filter import DoubleSlashCommentFilter
from mcp_software_ccopyright.yu.processors.html_comment_block_filter import HtmlCommentBlockFilter


class PhpProcessor(FileProcessorBase):
    expected_extensions = ['.php']

    def __init__(self):
        self.filters.append(BlankLineFilter())
        self.filters.append(CStyleCommentBlockFilter())
        self.filters.append(DoubleSlashCommentFilter())
        self.filters.append(HtmlCommentBlockFilter())
        return



import os

from mcp_software_ccopyright.yu.file_info import FileInfo
from mcp_software_ccopyright.yu.processors import FileProcessor


class ProjectProcessor(object):

    def __init__(self, project):
        self.project = project
        self.files = []
        self.file_processor = FileProcessor()
        return

    def process(self):
        self.find_files()
        self.sort_files()
        return self.files

    def find_files(self):
        for root, dirs, files in os.walk(self.project.source_root):
            if self.is_ignored(root):
                continue
            for file_name in files:
                file_path = os.path.join(root, file_name)
                if self.is_ignored(file_path):
                    continue
                name, extension = os.path.splitext(file_name)
                if self.is_interested_file(name, extension):
                    info = FileInfo(file_path, file_name, extension)
                    self.files.append(info)
        return

    def walk(self):
        return

    def is_ignored(self, path):
        """ Checks whether the specified path is ignored.
        """
        for ignore in self.project.ignore:
            if path.startswith(ignore):
                return True
        return False

    def is_interested_file(self, filename, extension):
        is_interested = True
        if len(self.project.filters) > 0:
            is_interested = False
            for filter in self.project.filters:
                if extension == filter:
                    is_interested = True
                    break

        if is_interested:
            return self.file_processor.has_interest(extension)
        else:
            return False

    def sort_files(self):
        return



from mcp_software_ccopyright.yu.processors import FileProcessorBase
from mcp_software_ccopyright.yu.processors import BlankLineFilter
from mcp_software_ccopyright.yu.processors.c_style_comment_block_filter import CStyleCommentBlockFilter
from mcp_software_ccopyright.yu.processors.double_slash_comment_filter import DoubleSlashCommentFilter


class JsProcessor(FileProcessorBase):
    expected_extensions = ['.js', '.jsx', '.vue', '.wpy']

    def __init__(self):
        self.filters.append(BlankLineFilter())
        self.filters.append(CStyleCommentBlockFilter())
        self.filters.append(DoubleSlashCommentFilter())
        return



from mcp_software_ccopyright.yu.processors import LineFilterBase


class CStyleCommentBlockFilter(LineFilterBase):
    """ Filters out C-style comment blocks with '/*' and '*/'
    """

    def filter(self, file, line):
        if file.is_in_comment_block:
            file.found_comment_line()
            if line.endswith('*/'):
                file.mark_comment_block_end()
            return line, True
        else:
            if line.startswith('/*'):
                file.found_comment_line()
                file.mark_comment_block_start()
                if line.endswith('*/'):
                    file.mark_comment_block_end()
                return line, True
        return line, False



import os.path
import json
import io


class ProjectInfo(object):
    """Represents all the information for a project.

    Attributes:
        source_root         Root directory of the source code folder for parsing.
        lines_to_extract    How many lines to extract, default to 3000
    """
    lines_to_extract = 3000
    project_root = ''
    source_root = ''
    lines_extracted = 0
    lines_skipped_comments = 0
    lines_skipped_blank = 0
    ignore = []
    filters = []

    def __init__(self, project_root, lines_to_extract):
        self.project_root = project_root
        self.lines_to_extract = float(lines_to_extract)
        self.parse_config()
        return

    def parse_config(self):
        """ Try to parse .ramileconfig.json file from the project root.
        """
        config_file = os.path.join(self.project_root, '.ramileconfig.json')
        if os.path.exists(config_file):
            with io.open(config_file, 'r') as file:
                config_data = json.load(file)
                print(config_data)
                self.__set_config_root(config_data)
                self.__set_config_ignore(config_data)
                self.__set_config_filters(config_data)
                self.__set_config_lines_to_extract(config_data)
        if self.source_root == '':
            self.source_root = self.project_root
        return

    def __set_config_filters(self, config_data):
        if 'filters' in config_data:
            self.filters = config_data['filters']

    def __set_config_root(self, config_data):
        if 'source_root' in config_data:
            self.source_root = os.path.join(
                self.project_root, config_data['source_root'])

    def __set_config_ignore(self, config_data):
        if 'ignore' in config_data:
            ignores = config_data['ignore']
            for ignore in ignores:
                self.ignore.append(os.path.join(self.source_root, ignore))
            print('Paths to be ignored: ', self.ignore)

    def __set_config_lines_to_extract(self, config_data):
        if 'lines_to_extract' in config_data:
            self.lines_to_extract = float(config_data['lines_to_extract'])

    def has_extracted_enough_lines(self):
        return self.lines_extracted >= self.lines_to_extract

    def get_output_file_path(self, output_filename):
        return os.path.join(self.project_root, output_filename)
